function particlesOrientations = f_crystalsGeneration3D(c, sampleVolume, particleRadius, particleHeight)
%
% determines grains in the mix according to given requirements
%
% particlesOrientations = f_crystalsGeneration3D(c, sampleVolume, GC_frac, GC_amount)
%
% input: c - volumetric fraction of grains in the composite [-]
%        sampleVolume - volume of the sample containing the aggregates
%        partecleRadius - radius of circumscribed circle
%        particleHeight - height of the modeled prisms
%
% output: particlesOrientations = 3 angles of defining the angle of rotation about axes
%
% written by Venca 9/8/2013

%% Calculation 

volume_of_particles = c*sampleVolume;
particlesOrientations = [];
actualVolume = 0;

while actualVolume < volume_of_particles
    actualVolume = actualVolume + 1.5*sqrt(3)*particleRadius^2*particleHeight;
    particlesOrientations = [particlesOrientations; 2*pi*rand, 2*pi*rand, 2*pi*rand];
end

suggested_volume_high = actualVolume;
suggested_volume_low = actualVolume-1.5*sqrt(3)*particleRadius^2*particleHeight;
if abs(sum(suggested_volume_high)-volume_of_particles) > abs(sum(suggested_volume_low)-volume_of_particles)
    particlesOrientations = particlesOrientations(1:end-1,:);
end


end
