function placedParticles = f_placeCrystals(particlesOrientations,cubeSizeRVE,minDistBetweenGrains,particleRadius,particleHeight,sphereRadius,coordinatesOfVertiecesOut,cAgg)

%%
% places crystals into the periodic RVE
% written by Venca 9/8/2013
%%

figureOut = false;
verticesRotated = zeros(12,3);
auxEdges = [1,2; 2,3; 3,4; 4,5; 5,6; 6,1; 7,8; 8,9; 9,10; 10,11; 11,12; 12,7; 1,7; 2,8; 3,9; 4,10; 5,11; 6,12]; % 18x2 auxiliary array of 18 lines connecting vertices [vertex 1, vertex 2...]

nParticles = size(particlesOrientations,1);
placedParticles = struct;

displayLimit = 1; % to inform about the progress
displayStep = 1;  % to inform about the progress
firstMessage = false; % to inform about the progress, must be false (not an option)

if figureOut
    cubeEdges = [0,0,0; cubeSizeRVE,0,0;
        cubeSizeRVE,0,0; cubeSizeRVE,cubeSizeRVE,0;
        cubeSizeRVE,cubeSizeRVE,0; 0,cubeSizeRVE,0;
        0,cubeSizeRVE,0; 0,0,0;
        0,0,cubeSizeRVE; cubeSizeRVE,0,cubeSizeRVE;
        cubeSizeRVE,0,cubeSizeRVE; cubeSizeRVE,cubeSizeRVE,cubeSizeRVE;
        cubeSizeRVE,cubeSizeRVE,cubeSizeRVE; 0,cubeSizeRVE,cubeSizeRVE;
        0,cubeSizeRVE,cubeSizeRVE; 0,0,cubeSizeRVE;
        0,0,0; 0,0,cubeSizeRVE;
        cubeSizeRVE,0,0; cubeSizeRVE,0,cubeSizeRVE;
        cubeSizeRVE,cubeSizeRVE,0; cubeSizeRVE,cubeSizeRVE,cubeSizeRVE;
        0,cubeSizeRVE,0; 0,cubeSizeRVE,cubeSizeRVE];
    
    for ce = 1:12
        plot3([cubeEdges(2*ce-1,1),cubeEdges(2*ce,1)],[cubeEdges(2*ce-1,2),cubeEdges(2*ce,2)],[cubeEdges(2*ce-1,3),cubeEdges(2*ce,3)])
        hold on
    end
end

if coordinatesOfVertiecesOut
    verticesCoordUncut = zeros(nParticles,49);
    verticesCoordCut = zeros(nParticles,49);
end

lastRow = 1;
for j = 1:nParticles
    
    % information about progress
    if ~firstMessage
        fprintf('Particles positioned:  0%%\n');
        firstMessage = true;
    end
    if j/nParticles*100 > displayLimit
        fprintf('\b\b\b\b')
        fprintf('%2.d%%\n', displayLimit);
        displayLimit = displayLimit+displayStep;
    end
    
    % placing the particles
    alpha = particlesOrientations(j,1);
    beta = particlesOrientations(j,2);
    gama = particlesOrientations(j,3);
    
    test = 0;
    while test == 0
        breakTest = 0;
        center = [cubeSizeRVE*rand,cubeSizeRVE*rand,cubeSizeRVE*rand];
        vertices = [particleRadius*cos(pi/3),   particleRadius*sin(pi/3),   -particleHeight/2;     % [x1, y1, z1;...; x12,y12,z12]
            particleRadius*cos(2*pi/3), particleRadius*sin(2*pi/3), -particleHeight/2;
            particleRadius*cos(pi),     particleRadius*sin(pi),     -particleHeight/2;
            particleRadius*cos(4*pi/3), particleRadius*sin(4*pi/3), -particleHeight/2;
            particleRadius*cos(5*pi/3), particleRadius*sin(5*pi/3), -particleHeight/2;
            particleRadius*cos(0),      particleRadius*sin(0),      -particleHeight/2;
            particleRadius*cos(pi/3),   particleRadius*sin(pi/3),   +particleHeight/2;     % upper part begins here (positive z-coords)
            particleRadius*cos(2*pi/3), particleRadius*sin(2*pi/3), +particleHeight/2;
            particleRadius*cos(pi),     particleRadius*sin(pi),     +particleHeight/2;
            particleRadius*cos(4*pi/3), particleRadius*sin(4*pi/3), +particleHeight/2;
            particleRadius*cos(5*pi/3), particleRadius*sin(5*pi/3), +particleHeight/2;
            particleRadius*cos(0),      particleRadius*sin(0),      +particleHeight/2];
        
        for v = 1:size(vertices,1)
            verticesRotated(v,:) = f_rotateCS_3D(vertices(v,:),[alpha,beta,gama]);
            for vv = 1:size(vertices,2)
                verticesRotated(v,vv) = verticesRotated(v,vv)+center(vv);
            end
        end
        
        checkBoundaries = zeros(6,12); % to check colision of vertices with boundaries of the sample
        % check whether the crystals go out of the sample
        for v = 1:length(verticesRotated)
            % crossing z = 0 plane
            if verticesRotated(v,3) < 0
                checkBoundaries(1,v) = 1;
            end
            % crossing z = cubeSizeRVE plane
            if verticesRotated(v,3) > cubeSizeRVE
                checkBoundaries(2,v) = 1;
            end
            % crossing y = cubeSizeRVE plane
            if verticesRotated(v,2) > cubeSizeRVE
                checkBoundaries(3,v) = 1;
            end
            % crossing x = cubeSizeRVE plane
            if verticesRotated(v,1) > cubeSizeRVE
                checkBoundaries(4,v) = 1;
            end
            % crossing y = 0 plane
            if verticesRotated(v,2) < 0
                checkBoundaries(5,v) = 1;
            end
            % crossing x = 0 plane
            if verticesRotated(v,1) < 0
                checkBoundaries(6,v) = 1;
            end
        end
        
        % check wheter some vertex goes out from the sample at the sample edge or corner
        A = sum(checkBoundaries);
        B = sum(checkBoundaries'); % to find what side of the RVE is crossed
        if ~isempty(A(A>1)) || length(find(B>0)) > 1 || sum(A) > 7 || sum(A) == 1 || sum(A) == 2 || sum(A) == 3 || sum(A) == 4
            test = 0;
            continue
        end
        if ~isempty(A(A>0))   % if some vertex goes from the sample -> duplication of spheres
            for d = 1:2
                if d == 1
                    placedParticles(lastRow).spheres = [center(1),center(2),center(3)];
                    if j == 1;
                        test = 1;
                    else
                        for k = 1:(lastRow-1)
                            deltaCenter = sqrt(sum((center-placedParticles(k).spheres).^2));
                            if 2*sphereRadius+minDistBetweenGrains > deltaCenter
                                test = 0;
                                breakTest = 1;
                                break
                            else
                                test = 1;
                            end
                        end
                    end
                    if breakTest == 1
                        break
                    end
                    
                    if B(1) > 0
                        placedParticles(lastRow+1).spheres = [center(1),center(2),center(3)+cubeSizeRVE];
                        normalToPlane = [0,0,1];
                        pointOnPlane = [0,0,0];
                        shiftToRVE = [0,0,cubeSizeRVE];
                    elseif B(2) > 0
                        placedParticles(lastRow+1).spheres = [center(1),center(2),center(3)-cubeSizeRVE];
                        normalToPlane = [0,0,-1];
                        pointOnPlane = [cubeSizeRVE,cubeSizeRVE,cubeSizeRVE];
                        shiftToRVE = [0,0,-cubeSizeRVE];
                    elseif B(3) > 0
                        placedParticles(lastRow+1).spheres = [center(1),center(2)-cubeSizeRVE,center(3)];
                        normalToPlane = [0,-1,0];
                        pointOnPlane = [cubeSizeRVE,cubeSizeRVE,cubeSizeRVE];
                        shiftToRVE = [0,-cubeSizeRVE,0];
                    elseif B(4) > 0
                        placedParticles(lastRow+1).spheres = [center(1)-cubeSizeRVE,center(2),center(3)];
                        normalToPlane = [-1,0,0];
                        pointOnPlane = [cubeSizeRVE,cubeSizeRVE,cubeSizeRVE];
                        shiftToRVE = [-cubeSizeRVE,0,0];
                    elseif B(5) > 0
                        placedParticles(lastRow+1).spheres = [center(1),center(2)+cubeSizeRVE,center(3)];
                        normalToPlane = [0,1,0];
                        pointOnPlane = [0,0,0];
                        shiftToRVE = [0,cubeSizeRVE,0];
                    elseif B(6) > 0
                        placedParticles(lastRow+1).spheres = [center(1)+cubeSizeRVE,center(2),center(3)];
                        normalToPlane = [1,0,0];
                        pointOnPlane = [0,0,0];
                        shiftToRVE = [cubeSizeRVE,0,0];
                    end
                    
                    verticesOut = find(A); % find what vertices are out of RVE
                    verticesOutCoords = verticesRotated(verticesOut,:);
                    verticesIn = find(A-ones(size(A)));
                    verticesInCoords = verticesRotated(verticesIn,:);
                    crossingPoints = []; % intersecting points with the boundaries [x,y,z]
                    disconnectedVertices = []; % vertices that were disconnected by the RVE boundary
                    
                    if coordinatesOfVertiecesOut % print the coordinates of vertices
                        verticesCoordUncut(j,1) = j;
                        for verticesToSave = 1:12
                            isOut = A(verticesToSave); % 1 = vertex is out
                            whatSide = find(B > 0); % code of the RVE side which is crossed
                            if isOut == 1
                                verticesCoordUncut(j,4*verticesToSave-2) = whatSide;
                            else
                                verticesCoordUncut(j,4*verticesToSave-2) = 0;
                            end
                            verticesCoordUncut(j,4*verticesToSave-1:4*verticesToSave+1) = verticesRotated(verticesToSave,:);
                        end
                        verticesCoordCut(j,1) = j;
                        for verticesToSave = 1:12
                            isOut = A(verticesToSave); % 1 = vertex is out
                            whatSide = find(B > 0); % code of the RVE side which is crossed
                            if isOut == 1
                                verticesCoordCut(j,4*verticesToSave-2) = whatSide;
                                verticesCoordCut(j,4*verticesToSave-1:4*verticesToSave+1) = verticesRotated(verticesToSave,:)+shiftToRVE;
                            else
                                verticesCoordUncut(j,4*verticesToSave-2) = 0;
                                verticesCoordCut(j,4*verticesToSave-1:4*verticesToSave+1) = verticesRotated(verticesToSave,:);
                            end
                        end
                    end
                    
                    for edge = 1:18 % check what lines intersect with the boundary
                        vertex_1_coords = verticesRotated(auxEdges(edge,1),:);
                        vertex_2_coords = verticesRotated(auxEdges(edge,2),:);
                        [pointOfIntersection, checkOfIntersection] = f_planeLineIntersect(normalToPlane,pointOnPlane,vertex_1_coords,vertex_2_coords);
                        if checkOfIntersection == 1
                            crossingPoints = [crossingPoints; pointOfIntersection];
                            disconnectedVertices = [disconnectedVertices; auxEdges(edge,1), auxEdges(edge,2)];
                        end
                    end
                    if size(crossingPoints,1) ~= 4
                        test = 0;
                        breakTest = 1;
                        break
                    end
                                
                    placedParticles(lastRow).crystals = [verticesInCoords; crossingPoints];
                    placedParticles(lastRow).areasOnCrystal = f_crystalCreateAreas(verticesRotated,verticesIn,disconnectedVertices,crossingPoints,[0,0,0]);
                    if figureOut
                        for av = 1:length(placedParticles(lastRow).areasOnCrystal)
                            p = patch(placedParticles(lastRow).areasOnCrystal(av).area(:,1),placedParticles(lastRow).areasOnCrystal(av).area(:,2),placedParticles(lastRow).areasOnCrystal(av).area(:,3),[0.5,0.5,0.9]);
                            hold on
                        end
                    end
                else % the second part of the "cut" crystal
                    if j == 1;
                        test = 1;
                        for voc = 1:size(verticesOutCoords,1)
                            verticesOutCoords(voc,:) = verticesOutCoords(voc,:)+shiftToRVE;
                        end
                        for cp = 1:size(crossingPoints,1)
                            crossingPoints(cp,:) = crossingPoints(cp,:)+shiftToRVE;
                        end
                        placedParticles(lastRow+1).crystals = [verticesOutCoords; crossingPoints];
                        placedParticles(lastRow+1).areasOnCrystal = f_crystalCreateAreas(verticesRotated,verticesOut,disconnectedVertices,crossingPoints,shiftToRVE);
                        if figureOut
                            for av = 1:length(placedParticles(lastRow+1).areasOnCrystal)
                                p = patch(placedParticles(lastRow+1).areasOnCrystal(av).area(:,1),placedParticles(lastRow+1).areasOnCrystal(av).area(:,2),placedParticles(lastRow+1).areasOnCrystal(av).area(:,3),[0.5,0.5,0.9]);
                                hold on
                            end
                        end
                        lastRow = lastRow+2;
                    else
                        for k = 1:(lastRow-1)
                            deltaCenter = sqrt(sum((placedParticles(lastRow+1).spheres-placedParticles(k).spheres).^2));
                            if 2*sphereRadius+minDistBetweenGrains > deltaCenter
                                test = 0;
                                breakTest = 1;
                                break
                            else
                                test = 1;
                                if k == lastRow-1
                                    for voc = 1:size(verticesOutCoords,1)
                                        verticesOutCoords(voc,:) = verticesOutCoords(voc,:)+shiftToRVE;
                                    end
                                    for cp = 1:size(crossingPoints,1)
                                        crossingPoints(cp,:) = crossingPoints(cp,:)+shiftToRVE;
                                    end
                                    placedParticles(lastRow+1).crystals = [verticesOutCoords; crossingPoints];
                                    placedParticles(lastRow+1).areasOnCrystal = f_crystalCreateAreas(verticesRotated,verticesOut,disconnectedVertices,crossingPoints,shiftToRVE);
                                    if figureOut
                                        for av = 1:length(placedParticles(lastRow+1).areasOnCrystal)
                                            p = patch(placedParticles(lastRow+1).areasOnCrystal(av).area(:,1),placedParticles(lastRow+1).areasOnCrystal(av).area(:,2),placedParticles(lastRow+1).areasOnCrystal(av).area(:,3),[0.5,0.5,0.9]);
                                            hold on
                                        end
                                    end
                                    lastRow = lastRow+2;
                                end
                            end
                        end
                    end
                    if breakTest == 1
                        break
                    end
                end
            end
            if breakTest == 1
                continue
            end
                    
        else % if there is no colision with the boundaries of the sample
            placedParticles(lastRow).spheres = [center(1),center(2),center(3)];
            
            if coordinatesOfVertiecesOut % print the coordinates of vertices
                verticesCoordUncut(j,1) = j;
                verticesCoordCut(j,1) = j;
                for verticesToSave = 1:12
                    verticesCoordUncut(j,4*verticesToSave-2) = 0;
                    verticesCoordCut(j,4*verticesToSave-2) = 0;
                    verticesCoordUncut(j,4*verticesToSave-1:4*verticesToSave+1) = verticesRotated(verticesToSave,:);
                    verticesCoordCut(j,4*verticesToSave-1:4*verticesToSave+1) = verticesRotated(verticesToSave,:);
                end
            end    
            
            if j == 1;
                test = 1;
                placedParticles(lastRow).crystals = verticesRotated;
                placedParticles(lastRow).areasOnCrystal = f_crystalCreateAreas(verticesRotated,1:12,[],[],[0,0,0]);
                if figureOut
                    for av = 1:length(placedParticles(lastRow).areasOnCrystal)
                        p = patch(placedParticles(lastRow).areasOnCrystal(av).area(:,1),placedParticles(lastRow).areasOnCrystal(av).area(:,2),placedParticles(lastRow).areasOnCrystal(av).area(:,3),[0.5,0.5,0.9]);
                        hold on
                    end
                end
                lastRow = lastRow+1;
            else
                for k = 1:(lastRow-1)
                    deltaCenter = sqrt(sum((center-placedParticles(k).spheres).^2));
                    if 2*sphereRadius+minDistBetweenGrains > deltaCenter
                        test = 0;
                        break
                    else
                        test = 1;
                        if k == lastRow-1
                            placedParticles(lastRow).crystals = verticesRotated;
                            placedParticles(lastRow).areasOnCrystal = f_crystalCreateAreas(verticesRotated,1:12,[],[],[0,0,0]);
                            if figureOut
                                for av = 1:length(placedParticles(lastRow).areasOnCrystal)
                                    p = patch(placedParticles(lastRow).areasOnCrystal(av).area(:,1),placedParticles(lastRow).areasOnCrystal(av).area(:,2),placedParticles(lastRow).areasOnCrystal(av).area(:,3),[0.5,0.5,0.9]);
                                    hold on
                                end
                            end
                            lastRow = lastRow+1;
                        end
                    end
                end
            end
        end
    end
end
fprintf('there were placed %d crystals (including just their fragments)\n', lastRow-1)

% if figureOut
%     ax = get(p,'Parent');
%     set(ax,'XLim', [0 cubeSizeRVE], 'YLim', [0 cubeSizeRVE], 'YLim', [0 cubeSizeRVE], 'PlotBoxAspectRatio', [1 1 1])
%     set(ax, 'Position',[0.1 0.1 0.8 0.8])
%     set(ax, 'XTick',[],'YTick',[],'ZTick',[])
%     box on;
%     view(26, 26);
%     axis([0 cubeSizeRVE 0 cubeSizeRVE 0 cubeSizeRVE]);
%     axis tight
%     axis equal
%     lightHandles = light;
%     set(lightHandles, 'Position', [cubeSizeRVE*2.6, cubeSizeRVE*0.5, cubeSizeRVE*1.1], 'Style', 'local');
%     lighting gouraud
%     
%     display('Saving figure...')
%     pngOutput = ['RVE_',num2str(nParticles),'_crystals.png'];
%     epsOutput = ['RVE_',num2str(nParticles),'_crystals.eps'];
%     eval(['print -dpng output/',pngOutput])
%     eval(['print -depsc2 output/',epsOutput])
% end

nameFileCut = ['outputCoordinates/',num2str(size(particlesOrientations,1)),'_crystals_c_',num2str(cAgg),'_cutCrystals.dat'];
nameFileUncut = ['outputCoordinates/',num2str(size(particlesOrientations,1)),'_crystals_c_',num2str(cAgg),'_uncutCrystals.dat'];

if coordinatesOfVertiecesOut
    dlmwrite(nameFileCut, verticesCoordCut, 'precision', 6);
    dlmwrite(nameFileUncut, verticesCoordUncut, 'precision', 6);
end

end
